// This file was generated by gir (https://github.com/gtk-rs/gir)
// from ../gir-files
// from ../pps-girs
// DO NOT EDIT

use crate::ffi;

glib::wrapper! {
    #[doc(alias = "PpsFormField")]
    pub struct FormField(Object<ffi::PpsFormField, ffi::PpsFormFieldClass>);

    match fn {
        type_ => || ffi::pps_form_field_get_type(),
    }
}

impl FormField {
    pub const NONE: Option<&'static FormField> = None;
}
