// This file was generated by gir (https://github.com/gtk-rs/gir)
// from ../gir-files
// from ../pps-girs
// DO NOT EDIT

use crate::ffi;
use glib::translate::*;

glib::wrapper! {
    #[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
    pub struct ViewSelection(Boxed<ffi::PpsViewSelection>);

    match fn {
        copy => |ptr| ffi::pps_view_selection_copy(mut_override(ptr)),
        free => |ptr| ffi::pps_view_selection_free(ptr),
        type_ => || ffi::pps_view_selection_get_type(),
    }
}
