// This file was generated by gir (https://github.com/gtk-rs/gir)
// from ../gir-files
// from ../pps-girs
// DO NOT EDIT

use crate::ffi;
use glib::translate::*;

glib::wrapper! {
    pub struct Point(BoxedInline<ffi::PpsPoint>);

    match fn {
        copy => |ptr| glib::gobject_ffi::g_boxed_copy(ffi::pps_point_get_type(), ptr as *mut _) as *mut ffi::PpsPoint,
        free => |ptr| glib::gobject_ffi::g_boxed_free(ffi::pps_point_get_type(), ptr as *mut _),
        type_ => || ffi::pps_point_get_type(),
    }
}

impl Point {
    #[doc(alias = "pps_point_new")]
    pub fn new() -> Point {
        assert_initialized_main_thread!();
        unsafe { from_glib_full(ffi::pps_point_new()) }
    }
}

impl Default for Point {
    fn default() -> Self {
        Self::new()
    }
}
