.content-view.document-page {
    box-shadow: color-mix(in srgb, var(--card-shade-color) 75%, transparent) 0px 2px 3px 1px, var(--card-shade-color) 0px 1px 3px 1px;
}

PpsView {
    background-color: var(--window-bg-color);
}

PpsView:selected {
    background-color: var(--accent-bg-color);
}

PpsView.document-page {
    background-color: white;
    padding: 0;
}
