// This file was generated by gir (https://github.com/gtk-rs/gir)
// from ../../gir-files
// from ../../pps-girs
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "PpsAnnotationEditingState", sizeof(PpsAnnotationEditingState), alignof(PpsAnnotationEditingState));
    printf("%s;%zu;%zu\n", "PpsAnnotationsContext", sizeof(PpsAnnotationsContext), alignof(PpsAnnotationsContext));
    printf("%s;%zu;%zu\n", "PpsAnnotationsContextClass", sizeof(PpsAnnotationsContextClass), alignof(PpsAnnotationsContextClass));
    printf("%s;%zu;%zu\n", "PpsAttachmentContext", sizeof(PpsAttachmentContext), alignof(PpsAttachmentContext));
    printf("%s;%zu;%zu\n", "PpsAttachmentContextClass", sizeof(PpsAttachmentContextClass), alignof(PpsAttachmentContextClass));
    printf("%s;%zu;%zu\n", "PpsAttachmentContextError", sizeof(PpsAttachmentContextError), alignof(PpsAttachmentContextError));
    printf("%s;%zu;%zu\n", "PpsDocumentModelClass", sizeof(PpsDocumentModelClass), alignof(PpsDocumentModelClass));
    printf("%s;%zu;%zu\n", "PpsHistory", sizeof(PpsHistory), alignof(PpsHistory));
    printf("%s;%zu;%zu\n", "PpsHistoryClass", sizeof(PpsHistoryClass), alignof(PpsHistoryClass));
    printf("%s;%zu;%zu\n", "PpsJob", sizeof(PpsJob), alignof(PpsJob));
    printf("%s;%zu;%zu\n", "PpsJobAnnots", sizeof(PpsJobAnnots), alignof(PpsJobAnnots));
    printf("%s;%zu;%zu\n", "PpsJobAnnotsClass", sizeof(PpsJobAnnotsClass), alignof(PpsJobAnnotsClass));
    printf("%s;%zu;%zu\n", "PpsJobAttachments", sizeof(PpsJobAttachments), alignof(PpsJobAttachments));
    printf("%s;%zu;%zu\n", "PpsJobAttachmentsClass", sizeof(PpsJobAttachmentsClass), alignof(PpsJobAttachmentsClass));
    printf("%s;%zu;%zu\n", "PpsJobClass", sizeof(PpsJobClass), alignof(PpsJobClass));
    printf("%s;%zu;%zu\n", "PpsJobExport", sizeof(PpsJobExport), alignof(PpsJobExport));
    printf("%s;%zu;%zu\n", "PpsJobExportClass", sizeof(PpsJobExportClass), alignof(PpsJobExportClass));
    printf("%s;%zu;%zu\n", "PpsJobFind", sizeof(PpsJobFind), alignof(PpsJobFind));
    printf("%s;%zu;%zu\n", "PpsJobFindClass", sizeof(PpsJobFindClass), alignof(PpsJobFindClass));
    printf("%s;%zu;%zu\n", "PpsJobFonts", sizeof(PpsJobFonts), alignof(PpsJobFonts));
    printf("%s;%zu;%zu\n", "PpsJobFontsClass", sizeof(PpsJobFontsClass), alignof(PpsJobFontsClass));
    printf("%s;%zu;%zu\n", "PpsJobLayers", sizeof(PpsJobLayers), alignof(PpsJobLayers));
    printf("%s;%zu;%zu\n", "PpsJobLayersClass", sizeof(PpsJobLayersClass), alignof(PpsJobLayersClass));
    printf("%s;%zu;%zu\n", "PpsJobLinks", sizeof(PpsJobLinks), alignof(PpsJobLinks));
    printf("%s;%zu;%zu\n", "PpsJobLinksClass", sizeof(PpsJobLinksClass), alignof(PpsJobLinksClass));
    printf("%s;%zu;%zu\n", "PpsJobLoad", sizeof(PpsJobLoad), alignof(PpsJobLoad));
    printf("%s;%zu;%zu\n", "PpsJobLoadClass", sizeof(PpsJobLoadClass), alignof(PpsJobLoadClass));
    printf("%s;%zu;%zu\n", "PpsJobPageData", sizeof(PpsJobPageData), alignof(PpsJobPageData));
    printf("%s;%zu;%zu\n", "PpsJobPageDataClass", sizeof(PpsJobPageDataClass), alignof(PpsJobPageDataClass));
    printf("%s;%zu;%zu\n", "PpsJobPageDataFlags", sizeof(PpsJobPageDataFlags), alignof(PpsJobPageDataFlags));
    printf("%s;%zu;%zu\n", "PpsJobPrint", sizeof(PpsJobPrint), alignof(PpsJobPrint));
    printf("%s;%zu;%zu\n", "PpsJobPrintClass", sizeof(PpsJobPrintClass), alignof(PpsJobPrintClass));
    printf("%s;%zu;%zu\n", "PpsJobPriority", sizeof(PpsJobPriority), alignof(PpsJobPriority));
    printf("%s;%zu;%zu\n", "PpsJobRenderTexture", sizeof(PpsJobRenderTexture), alignof(PpsJobRenderTexture));
    printf("%s;%zu;%zu\n", "PpsJobRenderTextureClass", sizeof(PpsJobRenderTextureClass), alignof(PpsJobRenderTextureClass));
    printf("%s;%zu;%zu\n", "PpsJobSave", sizeof(PpsJobSave), alignof(PpsJobSave));
    printf("%s;%zu;%zu\n", "PpsJobSaveClass", sizeof(PpsJobSaveClass), alignof(PpsJobSaveClass));
    printf("%s;%zu;%zu\n", "PpsJobSignatures", sizeof(PpsJobSignatures), alignof(PpsJobSignatures));
    printf("%s;%zu;%zu\n", "PpsJobSignaturesClass", sizeof(PpsJobSignaturesClass), alignof(PpsJobSignaturesClass));
    printf("%s;%zu;%zu\n", "PpsJobThumbnailTexture", sizeof(PpsJobThumbnailTexture), alignof(PpsJobThumbnailTexture));
    printf("%s;%zu;%zu\n", "PpsJobThumbnailTextureClass", sizeof(PpsJobThumbnailTextureClass), alignof(PpsJobThumbnailTextureClass));
    printf("%s;%zu;%zu\n", "PpsMetadataClass", sizeof(PpsMetadataClass), alignof(PpsMetadataClass));
    printf("%s;%zu;%zu\n", "PpsPageLayout", sizeof(PpsPageLayout), alignof(PpsPageLayout));
    printf("%s;%zu;%zu\n", "PpsSearchContext", sizeof(PpsSearchContext), alignof(PpsSearchContext));
    printf("%s;%zu;%zu\n", "PpsSearchContextClass", sizeof(PpsSearchContextClass), alignof(PpsSearchContextClass));
    printf("%s;%zu;%zu\n", "PpsSearchResult", sizeof(PpsSearchResult), alignof(PpsSearchResult));
    printf("%s;%zu;%zu\n", "PpsSearchResultClass", sizeof(PpsSearchResultClass), alignof(PpsSearchResultClass));
    printf("%s;%zu;%zu\n", "PpsSizingMode", sizeof(PpsSizingMode), alignof(PpsSizingMode));
    printf("%s;%zu;%zu\n", "PpsUndoContextClass", sizeof(PpsUndoContextClass), alignof(PpsUndoContextClass));
    printf("%s;%zu;%zu\n", "PpsUndoHandlerInterface", sizeof(PpsUndoHandlerInterface), alignof(PpsUndoHandlerInterface));
    printf("%s;%zu;%zu\n", "PpsView", sizeof(PpsView), alignof(PpsView));
    printf("%s;%zu;%zu\n", "PpsViewPresentation", sizeof(PpsViewPresentation), alignof(PpsViewPresentation));
    printf("%s;%zu;%zu\n", "PpsViewSelection", sizeof(PpsViewSelection), alignof(PpsViewSelection));
    return 0;
}
